//
//  FilePlayerController.m
//  PlayFileCocoa
//
//  Copyright 2011 JG Electronics. All rights reserved.
//	Remarks, reports to: www.jgelectronics.nl.
//
//	Derived from Apple's example program PlayFile and chapter 7 of the book Core Audio, authors Kevin Avila and Chris Adamson.
//	The application icon is from Vincent Garnier http://benjigarner.deviantart.com/art/Fruity-Apples-80805804
//
//	OSX10.6 and XCODE 4.0.1
//
/*
 Disclaimer: IMPORTANT: 
 This JG Electronics software is supplied to you by JG Electronics as FREE AND OPEN SOURCE SOFTWARE. 
 
 The JG Electronics Software is provided by JG Electronics on an "AS IS" basis.
 JG ELECTRONICS MAKES NO WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION
 THE IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS
 FOR A PARTICULAR PURPOSE, REGARDING THE JG ELECTRONICS SOFTWARE OR ITS USE AND
 OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS.
 
 IN NO EVENT SHALL JG ELECTRONICS BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL
 OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE, REPRODUCTION,
 MODIFICATION AND/OR DISTRIBUTION OF THE JG ELECTRONICS SOFTWARE, HOWEVER CAUSED
 AND WHETHER UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE),
 STRICT LIABILITY OR OTHERWISE, EVEN IF JG ELECTRONICS HAS BEEN ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
 */

#import "FilePlayerController.h"


@implementation FilePlayerController

	#pragma mark -
	#pragma mark === Accessor Methods ===
//	*********************************************************************************************************************************
//	archive
	@synthesize url;
//	slider values
	@synthesize amplitudeFlt;
//	time values
	@synthesize fpMaxTime;
	@synthesize fpRunTimeLeft, fpRunTimeGone;

	#pragma mark -
	#pragma mark === Internal Methods ===
//	*********************************************************************************************************************************
- (id)init
{
    self = [super init];
    if (self) {
		fpFilePlayerPtr = [FilePlayerModel new];
		amplitudeFlt = 0.75;
		fpIsRunning = NO;
		fpFileUrl = NO;
		fpNewFile = NO;
		fpFileRunTime = 0.0;
		fpRunTimeLeft = 0.0;
		fpRunTimeGone = 0.0;
		
	//	key value observing
		[self addObserver:self forKeyPath:@"amplitudeFlt" options:NSKeyValueObservingOptionOld context:nil];
	//	notifications
		NSNotificationCenter *	nc = [NSNotificationCenter defaultCenter];
		[nc addObserver:self selector:@selector(fpTimeMaxNotification:) name:@"timeMaxNotification" object:nil];
		[nc addObserver:self selector:@selector(fpTimerNotification:) name:@"timerNotification" object:nil];
		[nc addObserver:self selector:@selector(fpPlayBackEndNotification:) name:@"playbackEndNotification" object:nil];
	//
		BOOL succes = NO;
		succes = [fpFilePlayerPtr fpCreateAUGraph];
    }
    return self;
}

- (void)dealloc
{
	[fpFilePlayerPtr release];
	[url release];
	[fpMaxTime release];
	
    [super dealloc];
}

	#pragma mark -
	#pragma mark === Key Value Observing Methods ===
//	*********************************************************************************************************************************
- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
{
//	NSLog(@"observeValueForKeyPath");
	if([keyPath compare:@"amplitudeFlt"] == NSOrderedSame) {
		[fpFilePlayerPtr fpSetamplitude:[self amplitudeFlt]];
		[self fpSetMuteButtonImage];
	}
}

	#pragma mark -
	#pragma mark === Notification Methods ===
//	*********************************************************************************************************************************
- (void)fpTimeMaxNotification:(NSNotification *)notification
{
	NSLog(@"fpTimeMaxNotification notification");
	NSNumber *	maxTime = [[notification object]retain];
	[self setFpMaxTime:maxTime];
	[fpFileProgressInd setMaxValue:[fpMaxTime floatValue]];
	[self setFpRunTimeLeft:[fpMaxTime doubleValue]];
	[self setFpRunTimeGone:0.0];
	[maxTime release];
}
- (void)fpTimerNotification:(NSNotification *)notification
{
//	NSLog(@"fpTimerNotification Received");
	fpFileRunTime += 0.2;
	[self setFpRunTimeGone:fpFileRunTime];
	[fpFileProgressInd setDoubleValue:fpFileRunTime];
	[self setFpRunTimeLeft:[fpMaxTime doubleValue] - fpFileRunTime];
}

- (void)fpPlayBackEndNotification:(NSNotification *)notification
{
//	NSLog(@"fpPlayBackEndNotification Received");
	[fpFileProgressInd setDoubleValue:0.0];
	[self setFpRunTimeLeft:[fpMaxTime doubleValue]];
	[self setFpRunTimeGone:0.0];
	fpIsRunning = NO;
}

	#pragma mark -
	#pragma mark === Action Methods ===
//	*********************************************************************************************************************************
- (IBAction)fpReadButtons:(id)sender
{
	NSLog(@"fpReadButtons");
	BOOL	succes = NO;
	
	switch([sender tag])
	{
		case 1:		//	start to play file
			if(fpIsRunning == NO) {
				if(fpNewFile == YES) {
					succes = [fpFilePlayerPtr fpConfigureAudioFile:(CFURLRef)url];
					if(succes == NO) {
						NSLog(@"ConfigureAudioFile failed");
					}
					fpNewFile = NO;
					fpFileUrl = YES;
				}
				if(fpFileUrl == YES) {
					[fpFilePlayerPtr fpStartPlay];
					fpFileRunTime = 0.0;
					[fpFileProgressInd setDoubleValue:fpFileRunTime];
					fpIsRunning = YES;
				}
			}
			break;
		case 2:		//	pause
		
			break;
		case 3:		//	stop
			if(fpIsRunning == YES) {
				[fpFilePlayerPtr fpStopPlay];
				[fpFileProgressInd setDoubleValue:0.0];
				[self setFpRunTimeLeft:[fpMaxTime doubleValue]];
				[self setFpRunTimeGone:0.0];
				fpIsRunning = NO;
			}
			break;
	}
}

	#pragma mark -
	#pragma mark === Button and Slider Image Methods ===
//	*********************************************************************************************************************************
- (void)fpSetMuteButtonImage {
	
	if([self amplitudeFlt] < 0.06) {
		[fpSpeakerBevelButton setImage:[NSImage imageNamed:@"pl_volume0"]];
	}
	if(([self amplitudeFlt] > 0.05)&&([self amplitudeFlt] < 0.36)) {
		[fpSpeakerBevelButton setImage:[NSImage imageNamed:@"pl_volume1"]];
	}
	if(([self amplitudeFlt] > 0.35)&&([self amplitudeFlt] < 0.66)) {
		[fpSpeakerBevelButton setImage:[NSImage imageNamed:@"pl_volume2"]];
	}
	if([self amplitudeFlt] > 0.65) {
		[fpSpeakerBevelButton setImage:[NSImage imageNamed:@"pl_volume3"]];
	}
}

	#pragma mark -
	#pragma mark === Data Archive Methods ===
//	*********************************************************************************************************************************
- (IBAction)fpOpenSoundFile:(id)sender
{
    NSOpenPanel*	openPanel = [NSOpenPanel openPanel];
	NSArray *		fileTypes = [NSArray arrayWithObjects:@"AIFF", @"aif", @"aiff", @"aifc", @"wav", @"WAV", @"mp3", @"m4a", nil];
	
	[self setFpRunTimeLeft:0.0];
	[self setFpRunTimeGone:0.0];
	[openPanel setAllowedFileTypes:fileTypes];
    [openPanel setAllowsMultipleSelection:NO];
//	[openPanel beginSheetModalForWindow:fpWindow completionHandler:^(NSInteger result) {
	[openPanel beginWithCompletionHandler:^(NSInteger result) {
        if (result == NSOKButton) {
            [openPanel orderOut:self];
			NSArray *	fileURLsToOpen = [openPanel URLs];
			[self setUrl:[fileURLsToOpen objectAtIndex:0]];
			[fpWindow setRepresentedURL:[self url]];	//	sets icon
			fpNewFile = YES;
        }
    }];
}

@end
