//
//  FilePlayerModel.m
//  PlayFileCocoa
//
//  Copyright 2011 JG Electronics. All rights reserved.
//	Remarks, reports to: www.jgelectronics.nl.
//
//	Derived from Apple's example program PlayFile and chapter 7 of the book Core Audio, authors Kevin Avila and Chris Adamson.
//	The application icon is from Vincent Garnier http://benjigarner.deviantart.com/art/Fruity-Apples-80805804
//
//	OSX10.6 and XCODE 4.0.1
//
/*
 Disclaimer: IMPORTANT: 
 This JG Electronics software is supplied to you by JG Electronics as FREE AND OPEN SOURCE SOFTWARE. 
 
 The JG Electronics Software is provided by JG Electronics on an "AS IS" basis.
 JG ELECTRONICS MAKES NO WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION
 THE IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS
 FOR A PARTICULAR PURPOSE, REGARDING THE JG ELECTRONICS SOFTWARE OR ITS USE AND
 OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS.
 
 IN NO EVENT SHALL JG ELECTRONICS BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL
 OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE, REPRODUCTION,
 MODIFICATION AND/OR DISTRIBUTION OF THE JG ELECTRONICS SOFTWARE, HOWEVER CAUSED
 AND WHETHER UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE),
 STRICT LIABILITY OR OTHERWISE, EVEN IF JG ELECTRONICS HAS BEEN ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
 */

#import "FilePlayerModel.h"


@implementation FilePlayerModel

	#pragma mark -
	#pragma mark === Internal Methods ===
//	*********************************************************************************************************************************
- (id)init
{
    self = [super init];
    if (self) {
		fpTimer = [NSTimer scheduledTimerWithTimeInterval: 0.2 target: self selector: @selector(doTimer:) userInfo: nil repeats: YES];
		
		[[NSRunLoop currentRunLoop] addTimer: fpTimer forMode: NSModalPanelRunLoopMode];
		[[NSRunLoop currentRunLoop] addTimer: fpTimer forMode: NSEventTrackingRunLoopMode];
	//
		fpRunning = NO;
		fpFileDuration = 0.0;
		fpFileRunTime = 0.0;
    }
    
    return self;
}

- (void)dealloc
{
	AUGraphUninitialize(mGraph);
	AUGraphClose(mGraph);
	AudioFileClose(mInputFile);
    [super dealloc];
}

	#pragma mark -
	#pragma mark === Set Up Audio Graph Methods ===
//	*********************************************************************************************************************************
- (BOOL)fpCreateAUGraph
{
	AUNode						outputNode, fileNode;
	AudioComponentDescription	cd;
	OSStatus					error = noErr;
	
//	create AUGraph
	error = NewAUGraph(&mGraph);
	if(error != noErr) {
		NSLog(@"NewAUGraph failed");
		return NO;
	}
//	output node 
	cd.componentType = kAudioUnitType_Output;
	cd.componentSubType = kAudioUnitSubType_DefaultOutput;
	cd.componentManufacturer = kAudioUnitManufacturer_Apple;
	cd.componentFlags = 0;
	cd.componentFlagsMask = 0;
	error = AUGraphAddNode(mGraph, &cd, &outputNode);
	if(error != noErr) {
		NSLog(@"AUGraphAddNode outputNode failed");
		return NO;
	}
//	file player node
	cd.componentType = kAudioUnitType_Generator;
	cd.componentSubType = kAudioUnitSubType_AudioFilePlayer;
	error = AUGraphAddNode(mGraph, &cd, &fileNode);
	if(error != noErr) {
		NSLog(@"AUGraphAddNode fileNode failed");
		return NO;
	}
//	connect the output source of the file player AU to the input source of the output node
	error = AUGraphConnectNodeInput(mGraph, fileNode, 0, outputNode, 0);
	if(error != noErr) {
		NSLog(@"AUGraphConnectNodeInput failed");
		return NO;
	}
//	opening the graph opens all contained audio units but does not allocate any resources
	error = AUGraphOpen(mGraph);
	if(error != noErr) {
		NSLog(@"AUGraphOpen failed");
		return NO;
	}
//	get the reference to the AU object for the file player graph node
	error = AUGraphNodeInfo(mGraph, fileNode, NULL, &mFileAudio);
	if(error != noErr) {
		NSLog(@"AUGraphNodeInfo failed");
		return NO;
	}
//	get references to the AU object for the output graph node
	AUGraphNodeInfo(mGraph,outputNode, NULL, &mOutputAU);
	if(error != noErr) {
		NSLog(@"AUGraph mOutputAU Info failed");
		return NO;
	}
//	set our default value for volume
	AudioUnitSetParameter(mOutputAU, kHALOutputParam_Volume, kAudioUnitScope_Output, 0, 0.75, 0);
//	initialize the graph, causes resources to be allocated
	error = AUGraphInitialize(mGraph);
	if(error != noErr) {
		NSLog(@"AUGraphInitialize failed");
		return NO;
	}
	return YES;
}

	#pragma mark -
	#pragma mark === Audio File Methods ===
//	*********************************************************************************************************************************
- (BOOL)fpConfigureAudioFile:(CFURLRef)inURL
{
	NSLog(@"Start ConfigureAudioFile");
	UInt64			nPackets;
	UInt32			propsize = sizeof(nPackets);
	OSStatus		error= noErr;
	BOOL			succes = YES;

	error = AudioFileOpenURL (inURL, kAudioFileReadPermission, 0, &mInputFile);
	propsize = sizeof(AudioStreamBasicDescription);
	error = AudioFileGetProperty(mInputFile, kAudioFilePropertyDataFormat, &propsize, &mFileFormat);
	CFRelease(inURL);
	if(error != noErr) {
		NSLog(@"failed to get property fileFormat");
		return NO;
	}
	return succes;
}

- (BOOL)fpPrepareFileAU
{
	UInt64						nPackets;
	UInt32						propsize = sizeof(nPackets);
	ScheduledAudioFileRegion	rgn;
	OSStatus					error= noErr;
	BOOL						succes = YES;


//	tell the file player AU to load the file we want to play
	error = AudioUnitSetProperty(mFileAudio, kAudioUnitProperty_ScheduledFileIDs, kAudioUnitScope_Global, 0, &mInputFile, sizeof(mInputFile));
	if(error != noErr) {
		NSLog(@"AudioUnitSetProperty mInputFile failed");
	}
//	setting a scheduled file region for the AUFilePlayer
	error = AudioFileGetProperty(mInputFile, kAudioFilePropertyAudioDataPacketCount, &propsize, &nPackets);
	if(error != noErr) {
		NSLog(@"AudioFileGetProperty nPackets failed");
	}
//	tell the file player AU to play the entire file
	memset (&rgn.mTimeStamp, 0, sizeof(rgn.mTimeStamp));
	rgn.mTimeStamp.mFlags = kAudioTimeStampSampleTimeValid;
	rgn.mTimeStamp.mSampleTime = 0;
	rgn.mCompletionProc = NULL;
	rgn.mCompletionProcUserData = NULL;
	rgn.mAudioFile = mInputFile;
	rgn.mLoopCount = 1;
	rgn.mStartFrame = 0;
	rgn.mFramesToPlay = (UInt32)nPackets * mFileFormat.mFramesPerPacket;
	
	error = AudioUnitSetProperty(mFileAudio, kAudioUnitProperty_ScheduledFileRegion, kAudioUnitScope_Global, 0, &rgn, sizeof(rgn));
	if(error != noErr) {
		NSLog(@"AudioUnitSetProperty ScheduledAudioFileRegion failed");
	}
//	tell the file player AU when to start playing (-1 sample time means next render cycle)
	AudioTimeStamp startTime;
	memset (&startTime, 0, sizeof(startTime));
	startTime.mFlags = kAudioTimeStampSampleTimeValid;
	startTime.mSampleTime = -1;
	error = AudioUnitSetProperty(mFileAudio, kAudioUnitProperty_ScheduleStartTimeStamp, kAudioUnitScope_Global, 0, &startTime, sizeof(startTime));
	if(error != noErr) {
		NSLog(@"AudioUnitSetProperty startTime failed");
	}
//	calculate file playback time in seconds
	fpFileDuration = (nPackets * mFileFormat.mFramesPerPacket) / mFileFormat.mSampleRate;
	NSLog(@"file duration: %f secs\n",fpFileDuration);
	[[NSNotificationCenter defaultCenter] postNotificationName:@"timeMaxNotification" object:[NSNumber numberWithFloat:fpFileDuration]];
	
	return succes;
}

	#pragma mark -
	#pragma mark === Audio Control Methods ===
//	*********************************************************************************************************************************
- (void)fpStartPlay {
	OSStatus	error = noErr;
	
	error = AUGraphStart(mGraph);
	if(error != noErr) {
		NSLog(@"AUGraphStart failed");
	}
	error =  [self fpPrepareFileAU];
	if(error != YES) {
		NSLog(@"failed to fpPrepareFileAU");
	}
	fpRunning = YES;
	NSLog(@"fpIsRunning = YES");
// timing
	fpFileRunTime = 0;
}

- (void)fpStopPlay {
	OSStatus	error = noErr;
	
	error = AUGraphStop(mGraph);
	if(error != noErr) {
		NSLog(@"AUGraphStart failed");
	}
	fpRunning = NO;
	NSLog(@"fpIsRunning = NO");
}

- (void)fpSetamplitude:(float)amplFlt {
	OSStatus	error = noErr;
	
//	set the volume of the queue
	AudioUnitSetParameter(mOutputAU, kHALOutputParam_Volume, kAudioUnitScope_Output, 0, amplFlt, 0);
	if(error != noErr) {
		NSLog(@"AudioUnitSetParameter failed");
	}
}

- (void)doTimer:(NSTimer*)timer
{
//	NSLog(@"doTimer start");
	if(fpRunning == YES) {
		fpFileRunTime += 0.2;
		[[NSNotificationCenter defaultCenter] postNotificationName:@"timerNotification" object:nil];
		if(fpFileDuration < fpFileRunTime) {
			[self fpStopPlay];
			[[NSNotificationCenter defaultCenter] postNotificationName:@"playbackEndNotification" object:nil];
		}
	}
}

@end
